/* ====================================================================
   Copyright 2002-2004   Apache Software Foundation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */

package org.apache.poi.hssf.usermodel;

import junit.framework.TestCase;

import java.io.*;

/**
 * <p>Title: TestHSSFSheet_Hide_Clone</p>
 * <p>Description: Class for testing
 *                 1) cloning sheets which contains autofiltering
 *                 2) removing sheets which contains autofiltering
 *                 3) hidding sheets ( normal and strong hide )
 *    For first 2 tests , an Excel file is used as a template ; this file
 *    has only one sheet and this sheet has autofiltering set on first row .
 *    After cloning , the newly created Excel file should be manually opened .
 *    The expected result is : on the cloned sheet the autofiltering row should
 *    work properly .
 *    For third test ( hidding sheets ) a new Excel file with 2 sheets is created , then
 *    the second sheet is hided . Also this Excel file should be manually opened .
 *    The expected result is :
 *        - for both cases , the hidden sheet ( the second one ) shouldn't be visible.
 *        - if the sheet was hided using normal hide , then should be
 *          possible to make it visible from Excel interface : from menu Format->Sheet->Unhide
 *        - if the sheet  was hided using strong hide , then shouldn't be possible to make it
 *          visible using Excel interface ;  Format->Sheet->Unhide menu should be in disabled state
 * </p>
 * @author Dragos Buleandra ( dragos.buleandra@trade2b.ro )
 */

public class TestHSSFSheet_Hide_Clone extends TestCase
{
  public TestHSSFSheet_Hide_Clone( String name )
  {
     super(name);
  }

  protected void setUp()
  {
    String filename = System.getProperty("HSSF.testdata.path");
    if (filename == null)
    {
       System.setProperty("HSSF.testdata.path", "src/testcases/org/apache/poi/hssf/data");
    }
  }

  public void testCloneSheetWithAutofilter() throws IOException
  {
      System.out.println("\nTest no. 1 - Clone sheets with autofilter");
      String templateFile = System.getProperty("HSSF.testdata.path")+"/templateExcelWithAutofilter.xls";
      System.out.print("    Opening template  \""+templateFile+"\" ... ");
      HSSFWorkbook wb = new HSSFWorkbook(new FileInputStream(templateFile));
      System.out.println("done !");

      String resultFile   = System.getProperty("HSSF.testdata.path")+"/clonedExcelWithAutofilter.xls";
      System.out.print("    Cloning first sheet ... ");
      wb.cloneSheet(0);
      System.out.print("done !");
      //expecting to have 2 sheets now
      this.assertEquals("Number of sheets after cloning", 2, wb.getNumberOfSheets());

      //access cloned sheet and modify some properties , in order to be sure
      //that it is a valid Sheet object

      //preserve selection to first sheet
      wb.getSheetAt(1).setSelected(false);

      //rename it ( to be more user friendly )
      System.out.print("    Renaming cloned sheet from \""+wb.getSheetName(1)+"\" to \"Cloned sheet\" ... ");
      wb.setSheetName(1, "Cloned sheet");

      //expecting to have "Cloned sheet"
      this.assertEquals("Change cloned sheet's name", "Cloned sheet", wb.getSheetName(1).trim());
      System.out.println("done !");

      //write something on this sheet , in order to see if autofiltering is working
      //properly ( anyway , this can't be tested here but opening resulting file only )
      System.out.print("    Adding on this sheet 4 rows with 4 cells each ... ");
      HSSFSheet clonedSheet = wb.getSheetAt(1);
      for ( int i=1; i<=3; i++ )
      {
         clonedSheet.createRow(i);
      }
      //expecting to have 4 rows
      this.assertEquals("Added some rows to cloned sheet", 4, clonedSheet.getPhysicalNumberOfRows());

      for ( int i=1; i<=3; i++ )
      {
         HSSFRow currRow = clonedSheet.getRow(i);
         for ( int j=0; j<4; j++)
         {
            currRow.createCell((short)j).setCellValue("Cell ("+i+","+(j+1)+")");
         }
         //expecting to have 4 cells
         this.assertEquals("Added some cells to row "+currRow.getRowNum(), 4, currRow.getPhysicalNumberOfCells());
      }
      System.out.println("done !");

      //so , everything it's ok for now ; it remains for you to open the file and manually tests the autfilter
      System.out.println("\n    Everything it's ok since we've got so far -:) !\n"+
                         "    In order to complete the test , it remains for you to open the file and manually tests the autofilter : ");
      System.out.println("        File was saved in \""+resultFile+"\"");
      FileOutputStream fileOut = new FileOutputStream(resultFile);
      wb.write(fileOut);
      fileOut.close();
  }

  public void testRemoveSheetWithAutofilter() throws IOException
  {
      //on this test I suppose that cloning sheets it's already tested
      System.out.println("\nTest no. 2 - Remove sheets with autofilter");
      String templateFile = System.getProperty("HSSF.testdata.path")+"/templateExcelWithAutofilter.xls";
      System.out.print("    Opening template  \""+templateFile+"\" ... ");
      HSSFWorkbook wb = new HSSFWorkbook(new FileInputStream(templateFile) );
      System.out.println("done !");

      String resultFile   = System.getProperty("HSSF.testdata.path")+"/removedExcelWithAutofilter.xls";
      System.out.print("    Cloning first sheet ... ");
      wb.cloneSheet(0);
      System.out.println("done !");
      System.out.print("    Removing first sheet ... ");
      wb.removeSheetAt(0);
      //expecting to have 1 sheet now
      this.assertEquals("Number of sheets after deleting", 1, wb.getNumberOfSheets());
      System.out.println("done !");

      //write something on this sheet , in order to see if autofiltering is working
      //properly after removing the first sheet ( anyway , this can't be tested here but opening resulting file only )
      System.out.print("    Adding on this sheet 4 rows with 4 cells each ... ");
      HSSFSheet clonedSheet = wb.getSheetAt(0);
      for ( int i=1; i<=3; i++ )
      {
         clonedSheet.createRow(i);
      }
      //expecting to have 4 rows
      this.assertEquals("Added some rows to cloned sheet", 4, clonedSheet.getPhysicalNumberOfRows());

      for ( int i=1; i<=3; i++ )
      {
         HSSFRow currRow = clonedSheet.getRow(i);
         for ( int j=0; j<4; j++)
         {
            currRow.createCell((short)j).setCellValue("Cell ("+i+","+(j+1)+")");
         }
         //expecting to have 4 cells
         this.assertEquals("Added some cells to row "+currRow.getRowNum(), 4, currRow.getPhysicalNumberOfCells());
      }
      System.out.println("done !");

      //so , everything it's ok for now ; it remains for you to open the file and manually tests the autfilter
      System.out.println("\n    Everything it's ok since we've got so far -:) !\n"+
                         "    In order to complete the test , it remains for you to open the file and manually tests the autofilter : ");
      System.out.println("        File was saved in \""+resultFile+"\"");
      FileOutputStream fileOut = new FileOutputStream(resultFile);
      wb.write(fileOut);
      fileOut.close();
  }

  public void testNormalHideSheet() throws IOException
  {
    System.out.println("\nTest no. 3 - Hide sheet using normal hide");
    String resultFile   = System.getProperty("HSSF.testdata.path")+"/normalHideSheet.xls";
    HSSFWorkbook wb = new HSSFWorkbook();

    System.out.print("    Adding 2 sheets ... ");
    for ( int i=0; i<2; i++)
    {
      HSSFSheet fSheet = wb.createSheet();
    }
    //expecting to have 2 sheets now
    this.assertEquals("Number of sheets before hiding", 2, wb.getNumberOfSheets());

    System.out.println("done !");

    System.out.print("    Hide second sheet ( normal hide ) ... ");
    wb.hideSheet(1, true, false );
    //expecting to have 2 sheets now
    this.assertEquals("Number of sheets after hiding", 2, wb.getNumberOfSheets());
    System.out.println("done !");

    //so , everything it's ok for now ; it remains for you to open the file
    System.out.println("\n    Everything it's ok since we've got so far -:) !\n"+
                       "    In order to complete the test , it remains for you to open the file \n"+
                       "    and see if there's only one sheet then go to Format->Sheet->Unhide menu \n"+
                       "    and make the sheet visible again !");
    System.out.println("        File was saved in \""+resultFile+"\"");

    FileOutputStream fileOut = new FileOutputStream(resultFile);
    wb.write(fileOut);
    fileOut.close();
  }

  public void testStrongHideSheet() throws IOException
  {
    System.out.println("\nTest no. 4 - Hide sheet using strong hide");

    String resultFile   = System.getProperty("HSSF.testdata.path")+"/strongHideSheet.xls";
    HSSFWorkbook wb = new HSSFWorkbook();

    System.out.print("    Adding 2 sheets ... ");
    for ( int i=0; i<2; i++)
    {
      HSSFSheet fSheet = wb.createSheet();
    }
    //expecting to have 2 sheets now
    this.assertEquals("Number of sheets before hiding", 2, wb.getNumberOfSheets());
    System.out.println("done !");

    System.out.print("    Hide second sheet ( strong hide ) ... ");
    wb.hideSheet(1, true, true );
    //expecting to have 2 sheets now
    this.assertEquals("Number of sheets after hiding", 2, wb.getNumberOfSheets());
    System.out.println("done !");

    //so , everything it's ok for now ; it remains for you to open the file
    System.out.println("\n    Everything it's ok since we've got so far -:) !\n"+
                       "    In order to complete the test , it remains for you to open the file \n"+
                       "    and see if there's only one sheet then go to Format->Sheet->Unhide menu .\n"+
                       "    This menu should be unavailable ( in disabled state ) , which means that you \n"+
                       "    can't make the sheet visible again !");
    System.out.println("        File was saved in \""+resultFile+"\"");

    FileOutputStream fileOut = new FileOutputStream(resultFile);
    wb.write(fileOut);
    fileOut.close();
  }

  public static void main(String[] args)
  {
    junit.textui.TestRunner.run(TestHSSFSheet_Hide_Clone.class);
  }
}