/* ====================================================================
   Copyright 2002-2004   Apache Software Foundation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */

package org.apache.poi.hssf.usermodel;

import junit.framework.TestCase;

import java.io.*;

/**
 * <p>Title: TestOutliningGrouping</p>
 * <p>Description: Class for testing
 *                 1) outlining/grouping rows
 *                 2) outlining/grouping columns
 *    For both tests , an Excel file is generated . Each file contains 4 sheets :
 *        - on first sheet the rows/columns which are grouped/outlined are expanded and
 *          the summary row/column is on default state ( bellow for rows , on the right for columns)
 *        - on second sheet the rows/columns which are grouped/outlined are expanded and
 *          the summary row/column is above for rows , on the left for columns
 *        - on first sheet the rows/columns which are grouped/outlined are collapsed and
 *          the summary row/column is on default state ( bellow for rows , on the right for columns)
 *        - on second sheet the rows/columns which are grouped/outlined are collapsed and
 *          the summary row/column is above for rows , on the left for columns
 *    In order to see this , you should open the generated Excel files.
 * </p>
 * @author Dragos Buleandra ( dragos.buleandra@trade2b.ro )
 */

public class TestOutliningGrouping extends TestCase
{
  public TestOutliningGrouping( String name )
  {
    super(name);
  }

  protected void setUp()
  {
    String filename = System.getProperty("HSSF.testdata.path");
    if (filename == null)
    {
       System.setProperty("HSSF.testdata.path", "src/testcases/org/apache/poi/hssf/data");
    }
  }

  public void testOutlineRows() throws IOException
  {
    System.out.println("\nTest no. 1 - Outlining/grouping rows");
    String resultFile   = System.getProperty("HSSF.testdata.path")+"/OutliningGroupingRows.xls";
    HSSFWorkbook wb = new HSSFWorkbook();

    System.out.print("    Adding 4 sheets ... ");
    for ( int i=0; i<4; i++)
    {
      HSSFSheet fSheet = wb.createSheet();
    }
    System.out.println("done !");

    System.out.print("    Outlining rows in every sheet ... ");
    //first sheet
    HSSFSheet fSheet = wb.getSheetAt(0);
    fSheet.createOutlineRow((short)1, (short)1);
    fSheet.createOutlineRow((short)2, (short)2);
    fSheet.createOutlineRow((short)3, (short)2);
    fSheet.createOutlineRow((short)4, (short)1);
    //second sheet
    fSheet = wb.getSheetAt(1);
    fSheet.createOutlineRow((short)1, (short)1, false, false);
    fSheet.createOutlineRow((short)2, (short)2, false, false);
    fSheet.createOutlineRow((short)3, (short)2, false, false);
    fSheet.createOutlineRow((short)4, (short)1, false, false);
    //third sheet
    fSheet = wb.getSheetAt(2);
    fSheet.createOutlineRow((short)1, (short)1, true);
    fSheet.createOutlineRow((short)2, (short)2, true);
    fSheet.createOutlineRow((short)3, (short)2, true);
    fSheet.createOutlineRow((short)4, (short)1, true);
    //fourth sheet
    fSheet = wb.getSheetAt(3);
    fSheet.createOutlineRow((short)1, (short)1, true, false);
    fSheet.createOutlineRow((short)2, (short)2, true, false);
    fSheet.createOutlineRow((short)3, (short)2, true, false);
    fSheet.createOutlineRow((short)4, (short)1, true, false);

    System.out.println("done !");

    //so , everything it's ok for now ; it remains for you to open the file
    System.out.println("\n    Everything it's ok since we've got so far -:) !\n"+
                       "    In order to complete the test , it remains for you to open the file \n"+
                       "    and see if there are four sheets , as described !");
    System.out.println("        File was saved in \""+resultFile+"\"");

    FileOutputStream fileOut = new FileOutputStream(resultFile);
    wb.write(fileOut);
    fileOut.close();
  }

  public void testOutlineColumns() throws IOException
  {
    System.out.println("\nTest no. 1 - Outlining/grouping columns");
    String resultFile   = System.getProperty("HSSF.testdata.path")+"/OutliningGroupingColumns.xls";
    HSSFWorkbook wb = new HSSFWorkbook();

    System.out.print("    Adding 4 sheets ... ");
    for ( int i=0; i<4; i++)
    {
      HSSFSheet fSheet = wb.createSheet();
    }
    System.out.println("done !");

    System.out.print("    Outlining columns in every sheet ... ");
    //first sheet
    HSSFSheet fSheet = wb.getSheetAt(0);
    fSheet.createOutlineColumn((short)1, (short)1);
    fSheet.createOutlineColumn((short)2, (short)2);
    fSheet.createOutlineColumn((short)3, (short)2);
    fSheet.createOutlineColumn((short)4, (short)1);
    //second sheet
    fSheet = wb.getSheetAt(1);
    fSheet.createOutlineColumn((short)1, (short)1, false, false);
    fSheet.createOutlineColumn((short)2, (short)2, false, false);
    fSheet.createOutlineColumn((short)3, (short)2, false, false);
    fSheet.createOutlineColumn((short)4, (short)1, false, false);
    //third sheet
    fSheet = wb.getSheetAt(2);
    fSheet.createOutlineColumn((short)1, (short)1, true);
    fSheet.createOutlineColumn((short)2, (short)2, true);
    fSheet.createOutlineColumn((short)3, (short)2, true);
    fSheet.createOutlineColumn((short)4, (short)1, true);
    //fourth sheet
    fSheet = wb.getSheetAt(3);
    fSheet.createOutlineColumn((short)1, (short)1, true, false);
    fSheet.createOutlineColumn((short)2, (short)2, true, false);
    fSheet.createOutlineColumn((short)3, (short)2, true, false);
    fSheet.createOutlineColumn((short)4, (short)1, true, false);

    System.out.println("done !");

    //so , everything it's ok for now ; it remains for you to open the file
    System.out.println("\n    Everything it's ok since we've got so far -:) !\n"+
                       "    In order to complete the test , it remains for you to open the file \n"+
                       "    and see if there are four sheets , as described !");
    System.out.println("        File was saved in \""+resultFile+"\"");

    FileOutputStream fileOut = new FileOutputStream(resultFile);
    wb.write(fileOut);
    fileOut.close();
  }

  public static void main(String[] args)
  {
    junit.textui.TestRunner.run(TestOutliningGrouping.class);
  }

}